declare type Task<C, T> = (context: C) => T | Promise<T>;
declare type ExecutionResult<T> = {
    type: 'SUCCESS';
    returnValue: T;
} | {
    type: 'ERROR';
    caughtValue: any;
};
interface ExecutionMetadata {
    firstAttemptStartTime: Date;
    firstAttemptEndTime: Date;
    attemptNumber: number;
    isRetry: boolean;
    /**
     * Start time of this attempt
     */
    startTime: Date;
    /**
     * End time of this attempt
     */
    endTime: Date;
}
interface NextRunRequest {
    startTime: number | Date;
    isRetry: boolean;
}
declare type NextRunTimeEvaluator<C, T> = (result: ExecutionResult<T>, meta: ExecutionMetadata, context: C) => NextRunRequest | null;
export interface SingleInstanceTaskSchedulerOptions<C, T> {
    /**
     * A function that return the next run time of the task. This function will be called
     * after a task ended to evaluate the next run time. The returned value can be a delay
     * in milliseconds, or an absolute date time, or `null` which indicate no next run.
     */
    nextRunTimeEvaluator?: NextRunTimeEvaluator<C, T>;
}
export declare class SingleInstanceTaskScheduler<C = {}, T = void> {
    #private;
    constructor(task: Task<C, T>, initialContext: C, options?: SingleInstanceTaskSchedulerOptions<C, T>);
    get scheduled(): boolean;
    get running(): boolean;
    /**
     * Schedule the task to run after a given milliseconds or absolute date time. If
     * the task is already scheduled, it will be re-scheduled.
     * @param startTime Start time of the next run. A delay in milliseconds, or an absolute
     *   date time.
     */
    schedule(startTime: number | Date): void;
    cancelNextRun(): void;
    private scheduleWithResult;
    run(): void;
    runWaitResult(): Promise<T>;
}
interface OneTimeEvaluateOptions {
    type: 'ONE_TIME';
}
interface IntervalEvaluateOptions {
    type: 'RUN_START_TIME' | 'RUN_END_TIME';
    delay: number;
}
interface OnErrorEvaluateOptions {
    delay: number;
    attempt?: number;
}
interface BuildEvaluatorOptions {
    onSuccess: OneTimeEvaluateOptions | IntervalEvaluateOptions;
    onError?: OnErrorEvaluateOptions;
}
export declare function buildEvaluator<C, T>(options: BuildEvaluatorOptions): NextRunTimeEvaluator<C, T>;
export {};
