"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _task, _context, _nextRunTimeEvaluator, _nextRunTimer, _taskRunningPromise;
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildEvaluator = exports.SingleInstanceTaskScheduler = void 0;
const assert_1 = require("assert");
// eslint-disable-next-line @typescript-eslint/no-invalid-void-type
class SingleInstanceTaskScheduler {
    constructor(task, initialContext, options) {
        var _a;
        _task.set(this, void 0);
        _context.set(this, void 0);
        _nextRunTimeEvaluator.set(this, void 0);
        /**
         * `true` to indicate the timer will be set after task end
         */
        _nextRunTimer.set(this, null);
        _taskRunningPromise.set(this, null);
        __classPrivateFieldSet(this, _task, task);
        __classPrivateFieldSet(this, _context, {
            nextRunMetadata: {
                attemptNumber: 1,
                isRetry: false,
                firstAttempt: null
            },
            userContext: initialContext
        });
        __classPrivateFieldSet(this, _nextRunTimeEvaluator, (_a = options === null || options === void 0 ? void 0 : options.nextRunTimeEvaluator) !== null && _a !== void 0 ? _a : null);
    }
    get scheduled() {
        return (__classPrivateFieldGet(this, _nextRunTimer) !== null);
    }
    get running() {
        return __classPrivateFieldGet(this, _taskRunningPromise) !== null;
    }
    /**
     * Schedule the task to run after a given milliseconds or absolute date time. If
     * the task is already scheduled, it will be re-scheduled.
     * @param startTime Start time of the next run. A delay in milliseconds, or an absolute
     *   date time.
     */
    schedule(startTime) {
        let delay;
        if (startTime instanceof Date) {
            delay = startTime.getTime() - Date.now();
        }
        else {
            delay = startTime;
        }
        this.cancelNextRun();
        __classPrivateFieldSet(this, _nextRunTimer, setTimeout(this.run.bind(this), delay));
    }
    cancelNextRun() {
        if (__classPrivateFieldGet(this, _nextRunTimer) !== null) {
            if (__classPrivateFieldGet(this, _nextRunTimer) !== true) {
                clearTimeout(__classPrivateFieldGet(this, _nextRunTimer));
            }
            __classPrivateFieldSet(this, _nextRunTimer, null);
        }
    }
    scheduleWithResult(taskResult, startTime, endTime) {
        var _a, _b, _c, _d;
        let isNextRunRetry = false;
        if (__classPrivateFieldGet(this, _nextRunTimeEvaluator) == null) {
            __classPrivateFieldSet(this, _nextRunTimer, null);
        }
        else {
            const meta = __classPrivateFieldGet(this, _context).nextRunMetadata;
            const nextRunTime = __classPrivateFieldGet(this, _nextRunTimeEvaluator).call(this, taskResult, {
                firstAttemptStartTime: (_b = (_a = meta.firstAttempt) === null || _a === void 0 ? void 0 : _a.startTime) !== null && _b !== void 0 ? _b : startTime,
                firstAttemptEndTime: (_d = (_c = meta.firstAttempt) === null || _c === void 0 ? void 0 : _c.endTime) !== null && _d !== void 0 ? _d : endTime,
                attemptNumber: meta.attemptNumber,
                isRetry: meta.isRetry,
                startTime,
                endTime
            }, __classPrivateFieldGet(this, _context).userContext);
            if (nextRunTime == null) {
                __classPrivateFieldSet(this, _nextRunTimer, null);
            }
            else {
                if (__classPrivateFieldGet(this, _nextRunTimer) !== null) {
                    this.schedule(nextRunTime.startTime);
                }
                isNextRunRetry = nextRunTime.isRetry;
            }
        }
        const meta = __classPrivateFieldGet(this, _context).nextRunMetadata;
        meta.isRetry = isNextRunRetry;
        if (meta.isRetry) {
            if (meta.attemptNumber === 1) {
                meta.firstAttempt = {
                    startTime,
                    endTime
                };
            }
            meta.attemptNumber++;
        }
        else {
            meta.attemptNumber = 1;
            meta.firstAttempt = null;
        }
    }
    run() {
        __classPrivateFieldSet(this, _nextRunTimer, true);
        if (__classPrivateFieldGet(this, _taskRunningPromise) !== null) {
            return;
        }
        // In case of implementation error, we will just let it throw so that we can notice such error
        // eslint-disable-next-line @typescript-eslint/no-floating-promises
        __classPrivateFieldSet(this, _taskRunningPromise, (async () => {
            try {
                let taskResult;
                const startTime = new Date();
                try {
                    taskResult = {
                        type: 'SUCCESS',
                        returnValue: await __classPrivateFieldGet(this, _task).call(this, __classPrivateFieldGet(this, _context).userContext)
                    };
                }
                catch (error) {
                    taskResult = {
                        type: 'ERROR',
                        caughtValue: error
                    };
                }
                const endTime = new Date();
                this.scheduleWithResult(taskResult, startTime, endTime);
                if (taskResult.type === 'ERROR') {
                    throw taskResult.caughtValue;
                }
                return taskResult.returnValue;
            }
            finally {
                __classPrivateFieldSet(this, _taskRunningPromise, null);
            }
        })());
    }
    async runWaitResult() {
        this.run();
        if (__classPrivateFieldGet(this, _taskRunningPromise) === null) {
            assert_1.strict.fail('taskRunningPromise should not be null');
        }
        return await __classPrivateFieldGet(this, _taskRunningPromise);
    }
}
exports.SingleInstanceTaskScheduler = SingleInstanceTaskScheduler;
_task = new WeakMap(), _context = new WeakMap(), _nextRunTimeEvaluator = new WeakMap(), _nextRunTimer = new WeakMap(), _taskRunningPromise = new WeakMap();
function buildEvaluator(options) {
    return (result, meta) => {
        let request;
        if (result.type === 'SUCCESS') {
            const onSuccess = options.onSuccess;
            if (onSuccess.type === 'ONE_TIME') {
                request = null;
            }
            else if (onSuccess.type === 'RUN_START_TIME') {
                request = {
                    startTime: (meta.startTime.getTime() + onSuccess.delay) - Date.now(),
                    isRetry: false
                };
            }
            else if (onSuccess.type === 'RUN_END_TIME') {
                request = {
                    startTime: (meta.endTime.getTime() + onSuccess.delay) - Date.now(),
                    isRetry: false
                };
            }
            else {
                assert_1.strict.fail('Not implemented onSuccess.type');
            }
        }
        else if (result.type === 'ERROR') {
            if (options.onError === undefined ||
                (options.onError.attempt !== undefined && meta.attemptNumber >= options.onError.attempt)) {
                request = null;
            }
            else {
                request = {
                    startTime: (meta.endTime.getTime() + options.onError.delay) - Date.now(),
                    isRetry: true
                };
            }
        }
        else {
            assert_1.strict.fail('Not implemented onError.type');
        }
        return request;
    };
}
exports.buildEvaluator = buildEvaluator;
